@@ -1,44 +1,28 @@
 """Sphinx test suite utilities"""
 from __future__ import annotations
 
-import functools
+import contextlib
 import os
 import re
 import sys
 import warnings
-from typing import IO, TYPE_CHECKING, Any, Generator
+from typing import IO, TYPE_CHECKING, Any
 from xml.etree import ElementTree
 
 from docutils import nodes
-from docutils.nodes import Node
 from docutils.parsers.rst import directives, roles
 
 from sphinx import application, locale
 from sphinx.pycode import ModuleAnalyzer
-from sphinx.testing.path import path
-from sphinx.util.osutil import relpath
 
 if TYPE_CHECKING:
     from io import StringIO
 
-__all__ = [
-    'Struct', 'SphinxTestApp', 'SphinxTestAppWrapperForSkipBuilding',
-]
+    from docutils.nodes import Node
 
+    from sphinx.util._pathlib import _StrPath
 
-def assert_re_search(regex: re.Pattern, text: str, flags: int = 0) -> None:
-    if not re.search(regex, text, flags):
-        raise AssertionError(f'{regex!r} did not match {text!r}')
-
-
-def assert_not_re_search(regex: re.Pattern, text: str, flags: int = 0) -> None:
-    if re.search(regex, text, flags):
-        raise AssertionError(f'{regex!r} did match {text!r}')
-
-
-def assert_startswith(thing: str, prefix: str) -> None:
-    if not thing.startswith(prefix):
-        raise AssertionError(f'{thing!r} does not start with {prefix!r}')
+__all__ = 'SphinxTestApp', 'SphinxTestAppWrapperForSkipBuilding'
 
 
 def assert_node(node: Node, cls: Any = None, xpath: str = "", **kwargs: Any) -> None:
@@ -73,8 +57,10 @@ def assert_node(node: Node, cls: Any = None, xpath: str = "", **kwargs: Any) ->
             'The node%s does not have any attributes' % xpath
 
         for key, value in kwargs.items():
-            assert key in node, \
-                f'The node{xpath} does not have {key!r} attribute: {node!r}'
+            if key not in node:
+                if (key := key.replace('_', '-')) not in node:
+                    msg = f'The node{xpath} does not have {key!r} attribute: {node!r}'
+                    raise AssertionError(msg)
             assert node[key] == value, \
                 f'The node{xpath}[{key}] is not {value!r}: {node[key]!r}'
 
@@ -85,11 +71,6 @@ def etree_parse(path: str) -> Any:
         return ElementTree.parse(path)  # NoQA: S314  # using known data in tests
 
 
-class Struct:
-    def __init__(self, **kwargs: Any) -> None:
-        self.__dict__.update(kwargs)
-
-
 class SphinxTestApp(application.Sphinx):
     """
     A subclass of :class:`Sphinx` that runs on the test root, with some
@@ -101,8 +82,8 @@ class SphinxTestApp(application.Sphinx):
     def __init__(
         self,
         buildername: str = 'html',
-        srcdir: path | None = None,
-        builddir: path | None = None,
+        srcdir: _StrPath | None = None,
+        builddir: _StrPath | None = None,
         freshenv: bool = False,
         confoverrides: dict | None = None,
         status: IO | None = None,
@@ -111,25 +92,27 @@ class SphinxTestApp(application.Sphinx):
         docutilsconf: str | None = None,
         parallel: int = 0,
     ) -> None:
+        assert srcdir is not None
 
+        self.docutils_conf_path = srcdir / 'docutils.conf'
         if docutilsconf is not None:
-            (srcdir / 'docutils.conf').write_text(docutilsconf)
+            self.docutils_conf_path.write_text(docutilsconf, encoding='utf8')
 
         if builddir is None:
             builddir = srcdir / '_build'
 
         confdir = srcdir
         outdir = builddir.joinpath(buildername)
-        outdir.makedirs(exist_ok=True)
+        outdir.mkdir(parents=True, exist_ok=True)
         doctreedir = builddir.joinpath('doctrees')
-        doctreedir.makedirs(exist_ok=True)
+        doctreedir.mkdir(parents=True, exist_ok=True)
         if confoverrides is None:
             confoverrides = {}
         warningiserror = False
 
         self._saved_path = sys.path[:]
-        self._saved_directives = directives._directives.copy()  # type: ignore
-        self._saved_roles = roles._roles.copy()  # type: ignore
+        self._saved_directives = directives._directives.copy()  # type: ignore[attr-defined]
+        self._saved_roles = roles._roles.copy()  # type: ignore[attr-defined]
 
         self._saved_nodeclasses = {v for v in dir(nodes.GenericNodeVisitor)
                                    if v.startswith('visit_')}
@@ -147,13 +130,15 @@ class SphinxTestApp(application.Sphinx):
         locale.translators.clear()
         sys.path[:] = self._saved_path
         sys.modules.pop('autodoc_fodder', None)
-        directives._directives = self._saved_directives  # type: ignore
-        roles._roles = self._saved_roles  # type: ignore
+        directives._directives = self._saved_directives  # type: ignore[attr-defined]
+        roles._roles = self._saved_roles  # type: ignore[attr-defined]
         for method in dir(nodes.GenericNodeVisitor):
             if method.startswith('visit_') and \
                method not in self._saved_nodeclasses:
                 delattr(nodes.GenericNodeVisitor, 'visit_' + method[6:])
                 delattr(nodes.GenericNodeVisitor, 'depart_' + method[6:])
+        with contextlib.suppress(FileNotFoundError):
+            os.remove(self.docutils_conf_path)
 
     def __repr__(self) -> str:
         return f'<{self.__class__.__name__} buildername={self.builder.name!r}>'
@@ -177,32 +162,11 @@ class SphinxTestAppWrapperForSkipBuilding:
         return getattr(self.app, name)
 
     def build(self, *args: Any, **kwargs: Any) -> None:
-        if not self.app.outdir.listdir():  # type: ignore
+        if not os.listdir(self.app.outdir):
             # if listdir is empty, do build.
             self.app.build(*args, **kwargs)
             # otherwise, we can use built cache
 
 
-_unicode_literals_re = re.compile(r'u(".*?")|u(\'.*?\')')
-
-
-def find_files(root: str, suffix: str | None = None) -> Generator[str, None, None]:
-    for dirpath, _dirs, files in os.walk(root, followlinks=True):
-        dirpath = path(dirpath)
-        for f in [f for f in files if not suffix or f.endswith(suffix)]:
-            fpath = dirpath / f
-            yield relpath(fpath, root)
-
-
 def strip_escseq(text: str) -> str:
     return re.sub('\x1b.*?m', '', text)
-
-
-def simple_decorator(f):
-    """
-    A simple decorator that does nothing, for tests to use.
-    """
-    @functools.wraps(f)
-    def wrapper(*args, **kwargs):
-        return f(*args, **kwargs)
-    return wrapper
